@@ -4,6 +4,7 @@ from typing import Tuple
 
 from .exceptions import InverterError
 from .inverter import Inverter
+from .inverter import OperationMode
 from .inverter import SensorKind as Kind
 from .protocol import ProtocolCommand, ModbusReadCommand, ModbusWriteCommand, ModbusWriteMultiCommand
 from .sensor import *
@@ -16,18 +17,18 @@ class DT(Inverter):
         Timestamp("timestamp", 0, "Timestamp"),
         Voltage("vpv1", 6, "PV1 Voltage", Kind.PV),
         Current("ipv1", 8, "PV1 Current", Kind.PV),
-        Calculated("ppv1", 0,
-                   lambda data, _: round(read_voltage(data, 6) * read_current(data, 8)),
+        Calculated("ppv1",
+                   lambda data: round(read_voltage(data, 6) * read_current(data, 8)),
                    "PV1 Power", "W", Kind.PV),
         Voltage("vpv2", 10, "PV2 Voltage", Kind.PV),
         Current("ipv2", 12, "PV2 Current", Kind.PV),
-        Calculated("ppv2", 0,
-                   lambda data, _: round(read_voltage(data, 10) * read_current(data, 12)),
+        Calculated("ppv2",
+                   lambda data: round(read_voltage(data, 10) * read_current(data, 12)),
                    "PV2 Power", "W", Kind.PV),
         Voltage("vpv3", 14, "PV3 Voltage", Kind.PV),
         Current("ipv3", 16, "PV3 Current", Kind.PV),
-        Calculated("ppv3", 0,
-                   lambda data, _: round(read_voltage(data, 14) * read_current(data, 16)),
+        Calculated("ppv3",
+                   lambda data: round(read_voltage(data, 14) * read_current(data, 16)),
                    "PV3 Power", "W", Kind.PV),
         # Voltage("vpv4", 14, "PV4 Voltage", Kind.PV),
         # Current("ipv4", 16, "PV4 Current", Kind.PV),
@@ -47,14 +48,14 @@ class DT(Inverter):
         Frequency("fgrid1", 48, "On-grid L1 Frequency", Kind.AC),
         Frequency("fgrid2", 50, "On-grid L2 Frequency", Kind.AC),
         Frequency("fgrid3", 52, "On-grid L3 Frequency", Kind.AC),
-        Calculated("pgrid1", 0,
-                   lambda data, _: round(read_voltage(data, 36) * read_current(data, 42)),
+        Calculated("pgrid1",
+                   lambda data: round(read_voltage(data, 36) * read_current(data, 42)),
                    "On-grid L1 Power", "W", Kind.AC),
-        Calculated("pgrid2", 0,
-                   lambda data, _: round(read_voltage(data, 38) * read_current(data, 44)),
+        Calculated("pgrid2",
+                   lambda data: round(read_voltage(data, 38) * read_current(data, 44)),
                    "On-grid L2 Power", "W", Kind.AC),
-        Calculated("pgrid3", 0,
-                   lambda data, _: round(read_voltage(data, 40) * read_current(data, 46)),
+        Calculated("pgrid3",
+                   lambda data: round(read_voltage(data, 40) * read_current(data, 46)),
                    "On-grid L3 Power", "W", Kind.AC),
         Integer("xx54", 54, "Unknown sensor@54"),
         Power("ppv", 56, "PV Power", Kind.PV),
@@ -77,7 +78,7 @@ class DT(Inverter):
         Energy4("e_total", 90, "Total PV Generation", Kind.PV),
         Long("h_total", 94, "Hours Total", "h", Kind.PV),
         Integer("safety_country", 98, "Safety Country code", "", Kind.AC),
-        Enum2("safety_country_label", 98, SAFETY_COUNTRIES_ET, "Safety Country", "", Kind.AC),
+        Enum2("safety_country_label", 98, SAFETY_COUNTRIES_ET, "Safety Country", Kind.AC),
         Integer("xx100", 100, "Unknown sensor@100"),
         Integer("xx102", 102, "Unknown sensor@102"),
         Integer("xx104", 104, "Unknown sensor@104"),
@@ -138,10 +139,10 @@ class DT(Inverter):
         except:
             print("No model name sent from the inverter.")
         self.serial_number = response[6:22].decode("ascii")
-        self.dsp1_sw_version = read_unsigned_int(response, 66)
-        self.dsp2_sw_version = read_unsigned_int(response, 68)
-        self.arm_sw_version = read_unsigned_int(response, 70)
-        self.software_version = "{}.{}.{:02x}".format(self.dsp1_sw_version, self.dsp2_sw_version, self.arm_sw_version)
+        self.dsp1_version = read_unsigned_int(response, 66)
+        self.dsp2_version = read_unsigned_int(response, 68)
+        self.arm_version = read_unsigned_int(response, 70)
+        self.firmware = "{}.{}.{:02x}".format(self.dsp1_version, self.dsp2_version, self.arm_version)
 
         if any(model in self.serial_number for model in ["DSN", "MSU", "PSB", "PSC"]):
             # this is single phase inverter, filter out all L2 and L3 sensors
@@ -194,10 +195,10 @@ class DT(Inverter):
         if export_limit >= 0 or export_limit <= 100:
             return await self.write_setting('grid_export_limit', export_limit)
 
-    async def get_operation_mode(self) -> int:
+    async def get_operation_mode(self) -> OperationMode:
         raise InverterError("Operation not supported.")
 
-    async def set_operation_mode(self, operation_mode: int, eco_mode_power: int = 100) -> None:
+    async def set_operation_mode(self, operation_mode: OperationMode, eco_mode_power: int = 100) -> None:
         raise InverterError("Operation not supported.")
 
     async def get_ongrid_battery_dod(self) -> int:
