@@ -2,6 +2,7 @@ from typing import Tuple
 
 from .exceptions import InverterError
 from .inverter import Inverter
+from .inverter import OperationMode
 from .inverter import SensorKind as Kind
 from .protocol import ProtocolCommand, Aa55ProtocolCommand
 from .sensor import *
@@ -17,30 +18,32 @@ class ES(Inverter):
     __sensors: Tuple[Sensor, ...] = (
         Voltage("vpv1", 0, "PV1 Voltage", Kind.PV),  # modbus 0x500
         Current("ipv1", 2, "PV1 Current", Kind.PV),
-        Calculated("ppv1", 0, lambda data, _: round(read_voltage(data, 0) * read_current(data, 2)), "PV1 Power", "W",
-                   Kind.PV),
+        Calculated("ppv1",
+                   lambda data: round(read_voltage(data, 0) * read_current(data, 2)),
+                   "PV1 Power", "W", Kind.PV),
         Byte("pv1_mode", 4, "PV1 Mode code", "", Kind.PV),
-        Enum("pv1_mode_label", 4, PV_MODES, "PV1 Mode", "", Kind.PV),
+        Enum("pv1_mode_label", 4, PV_MODES, "PV1 Mode", Kind.PV),
         Voltage("vpv2", 5, "PV2 Voltage", Kind.PV),
         Current("ipv2", 7, "PV2 Current", Kind.PV),
-        Calculated("ppv2", 0, lambda data, _: round(read_voltage(data, 5) * read_current(data, 7)), "PV2 Power", "W",
-                   Kind.PV),
+        Calculated("ppv2",
+                   lambda data: round(read_voltage(data, 5) * read_current(data, 7)),
+                   "PV2 Power", "W", Kind.PV),
         Byte("pv2_mode", 9, "PV2 Mode code", "", Kind.PV),
-        Enum("pv2_mode_label", 9, PV_MODES, "PV2 Mode", "", Kind.PV),
-        Calculated("ppv", 0,
-                   lambda data, _: round(read_voltage(data, 0) * read_current(data, 2)) + round(
+        Enum("pv2_mode_label", 9, PV_MODES, "PV2 Mode", Kind.PV),
+        Calculated("ppv",
+                   lambda data: round(read_voltage(data, 0) * read_current(data, 2)) + round(
                        read_voltage(data, 5) * read_current(data, 7)),
                    "PV Power", "W", Kind.PV),
         Voltage("vbattery1", 10, "Battery Voltage", Kind.BAT),  # modbus 0x506
         # Voltage("vbattery2", 12, "Battery Voltage 2", Kind.BAT),
         Integer("battery_status", 14, "Battery Status", "", Kind.BAT),
         Temp("battery_temperature", 16, "Battery Temperature", Kind.BAT),
-        Calculated("ibattery1", 18,
-                   lambda data, _: abs(read_current(data, 18)) * (-1 if read_byte(data, 30) == 3 else 1),
+        Calculated("ibattery1",
+                   lambda data: abs(read_current(data, 18)) * (-1 if read_byte(data, 30) == 3 else 1),
                    "Battery Current", "A", Kind.BAT),
         # round(vbattery1 * ibattery1),
-        Calculated("pbattery1", 0,
-                   lambda data, _: abs(
+        Calculated("pbattery1",
+                   lambda data: abs(
                        round(read_voltage(data, 10) * read_current(data, 18))
                    ) * (-1 if read_byte(data, 30) == 3 else 1),
                    "Battery Power", "W", Kind.BAT),
@@ -52,25 +55,25 @@ class ES(Inverter):
         # Byte("cbattery3", 28, "Battery State of Charge 3", "%", Kind.BAT),
         Byte("battery_soh", 29, "Battery State of Health", "%", Kind.BAT),
         Byte("battery_mode", 30, "Battery Mode code", "", Kind.BAT),
-        Enum("battery_mode_label", 30, BATTERY_MODES_ET, "Battery Mode", "", Kind.BAT),
+        Enum("battery_mode_label", 30, BATTERY_MODES_ET, "Battery Mode", Kind.BAT),
         Integer("battery_warning", 31, "Battery Warning", "", Kind.BAT),
         Byte("meter_status", 33, "Meter Status code", "", Kind.AC),
         Voltage("vgrid", 34, "On-grid Voltage", Kind.AC),
         Current("igrid", 36, "On-grid Current", Kind.AC),
-        Calculated("pgrid", 38,
-                   lambda data, _: abs(read_power2(data, 38)) * (-1 if read_byte(data, 80) == 2 else 1),
+        Calculated("pgrid",
+                   lambda data: abs(read_bytes2(data, 38)) * (-1 if read_byte(data, 80) == 2 else 1),
                    "On-grid Export Power", "W", Kind.AC),
         Frequency("fgrid", 40, "On-grid Frequency", Kind.AC),
         Byte("grid_mode", 42, "Work Mode code", "", Kind.GRID),
-        Enum("grid_mode_label", 42, WORK_MODES_ES, "Work Mode", "", Kind.GRID),
+        Enum("grid_mode_label", 42, WORK_MODES_ES, "Work Mode", Kind.GRID),
         Voltage("vload", 43, "Back-up Voltage", Kind.UPS),  # modbus 0x51b
         Current("iload", 45, "Back-up Current", Kind.UPS),
         Power("pload", 47, "On-grid Power", Kind.AC),
         Frequency("fload", 49, "Back-up Frequency", Kind.UPS),
         Byte("load_mode", 51, "Load Mode code", "", Kind.AC),
-        Enum("load_mode_label", 51, LOAD_MODES, "Load Mode", "", Kind.AC),
+        Enum("load_mode_label", 51, LOAD_MODES, "Load Mode", Kind.AC),
         Byte("work_mode", 52, "Energy Mode code", "", Kind.AC),
-        Enum("work_mode_label", 52, ENERGY_MODES, "Energy Mode", "", Kind.AC),
+        Enum("work_mode_label", 52, ENERGY_MODES, "Energy Mode", Kind.AC),
         Temp("temperature", 53, "Inverter Temperature"),
         Long("error_codes", 55, "Error Codes"),
         Energy4("e_total", 59, "Total PV Generation", Kind.PV),
@@ -82,21 +85,17 @@ class ES(Inverter):
         Byte("effective_work_mode", 77, "Effective Work Mode code"),
         Integer("effective_relay_control", 78, "Effective Relay Control", "", None),
         Byte("grid_in_out", 80, "On-grid Mode code", "", Kind.GRID),
-        Calculated("grid_in_out_label", 0,
-                   lambda data, _: GRID_IN_OUT_MODES.get(read_byte(data, 80)),
-                   "On-grid Mode", "", Kind.GRID),
+        Enum("grid_in_out_label", 80, GRID_IN_OUT_MODES, "On-grid Mode", Kind.GRID),
         Power("pback_up", 81, "Back-up Power", Kind.UPS),
         # pload + pback_up
-        Calculated("plant_power", 0,
-                   lambda data, _: round(read_power2(data, 47) + read_power2(data, 81)),
+        Calculated("plant_power",
+                   lambda data: round(read_bytes2(data, 47) + read_bytes2(data, 81)),
                    "Plant Power", "W", Kind.AC),
         Decimal("meter_power_factor", 83, 1000, "Meter Power Factor", "", Kind.GRID),  # modbus 0x531
         Integer("xx85", 85, "Unknown sensor@85"),
         Integer("xx87", 87, "Unknown sensor@87"),
         Long("diagnose_result", 89, "Diag Status Code"),
-        Calculated("diagnose_result_label", 0,
-                   lambda data, _: decode_bitmap(read_bytes4(data, 89), DIAG_STATUS_CODES),
-                   "Diag Status", ""),
+        EnumBitmap4("diagnose_result_label", 89, DIAG_STATUS_CODES, "Diag Status"),
         # Energy4("e_total_exp", 93, "Total Energy (export)", Kind.GRID),
         # Energy4("e_total_imp", 97, "Total Energy (import)", Kind.GRID),
         # Voltage("vpv3", 101, "PV3 Voltage", Kind.PV),  # modbus 0x500
@@ -110,29 +109,29 @@ class ES(Inverter):
         # Energy4("e_bat_discharge_total", 117, "Total Battery Discharge", Kind.BAT),
 
         # ppv1 + ppv2 + pbattery - pgrid
-        Calculated("house_consumption", 0,
-                   lambda data, _:
+        Calculated("house_consumption",
+                   lambda data:
                    round(read_voltage(data, 0) * read_current(data, 2)) +
                    round(read_voltage(data, 5) * read_current(data, 7)) +
                    (abs(round(read_voltage(data, 10) * read_current(data, 18))) *
                     (-1 if read_byte(data, 30) == 3 else 1)) -
-                   (abs(read_power2(data, 38)) * (-1 if read_byte(data, 80) == 2 else 1)),
+                   (abs(read_bytes2(data, 38)) * (-1 if read_byte(data, 80) == 2 else 1)),
                    "House Consumption", "W", Kind.AC),
     )
 
     __settings: Tuple[Sensor, ...] = (
-        Integer("charge_power_limit", 4, "Charge Power Limit Value"),
-        Integer("discharge_power_limit", 10, "Disharge Power Limit Value"),
-        Byte("relay_control", 13, "Relay Control"),
-        Byte("off-grid_charge", 15, "Off-grid Charge"),
-        Byte("shadow_scan", 17, "Shadow Scan"),
-        Integer("backflow_state", 18, "Backflow State"),
+        EcoModeEs("eco_mode_charge", 0, "Eco Mode Charge"),
+        EcoModeEs("eco_mode_discharge", 6, "Eco Mode Discharge"),
+        Integer("backup_supply", 12, "Backup Supply"),
+        Integer("off-grid_charge", 14, "Off-grid Charge"),
+        Integer("shadow_scan", 16, "Shadow Scan"),
+        Integer("grid_export", 18, "Grid Export Enabled"),
         Integer("capacity", 22, "Capacity"),
         Integer("charge_v", 24, "Charge Voltage", "V"),
         Integer("charge_i", 26, "Charge Current", "A", ),
         Integer("discharge_i", 28, "Discharge Current", "A", ),
         Integer("discharge_v", 30, "Discharge Voltage", "V"),
-        Calculated("dod", 32, lambda data, _: 100 - read_bytes2(data, 32), "Depth of Discharge", "%"),
+        Calculated("dod", lambda data: 100 - read_bytes2(data, 32), "Depth of Discharge", "%"),
         Integer("battery_activated", 34, "Battery Activated"),
         Integer("bp_off_grid_charge", 36, "BP Off-grid Charge"),
         Integer("bp_pv_discharge", 38, "BP PV Discharge"),
@@ -143,21 +142,18 @@ class ES(Inverter):
         Integer("work_mode", 66, "Work Mode"),
         Integer("grid_quality_check", 68, "Grid Quality Check"),
 
-        # emulated settings
+        # emulated settings for API compatibility with ET inverters
         EcoMode("eco_mode_1", 0, "Eco Mode Power Group 1"),
-        EcoMode("eco_mode_2", 0, "Eco Mode Power Group 2"),
-        EcoMode("eco_mode_3", 0, "Eco Mode Power Group 3"),
-        EcoMode("eco_mode_4", 0, "Eco Mode Power Group 4"),
     )
 
     async def read_device_info(self):
         response = await self._read_from_socket(self._READ_DEVICE_VERSION_INFO)
-        self.arm_version = response[7:12].decode("ascii").rstrip()
+        self.arm_firmware = response[7:12].decode("ascii").rstrip()
         self.model_name = response[12:22].decode("ascii").rstrip()
         self.serial_number = response[38:54].decode("ascii")
         self.software_version = response[58:70].decode("ascii")
-        if len(self.arm_version) >= 5:
-            self.arm_sw_version = int(self.arm_version[4], base=36)
+        if len(self.arm_firmware) >= 5:
+            self.arm_version = int(self.arm_firmware[4], base=36)
 
     async def read_runtime_data(self, include_unknown_sensors: bool = False) -> Dict[str, Any]:
         raw_data = await self._read_from_socket(self._READ_DEVICE_RUNNING_DATA)
@@ -170,13 +166,14 @@ class ES(Inverter):
             # There does not seem to be time setting/sensor evailable (or is not known)
             return datetime.now()
         elif setting_id == 'eco_mode_1':
-            return None
-        elif setting_id == 'eco_mode_2':
-            return None
-        elif setting_id == 'eco_mode_3':
-            return None
-        elif setting_id == 'eco_mode_4':
-            return None
+            # Emulated ET like setting
+            all_settings = await self.read_settings_data()
+            eco_charge = all_settings.get('eco_mode_charge')
+            eco_discharge = all_settings.get('eco_mode_discharge')
+            if eco_charge.is_eco_charge_mode():
+                return eco_charge.asEcoMode(True)
+            else:
+                return eco_discharge.asEcoMode(False)
         else:
             all_settings = await self.read_settings_data()
             return all_settings.get(setting_id)
@@ -186,14 +183,6 @@ class ES(Inverter):
             await self._read_from_socket(
                 Aa55ProtocolCommand("030206" + Timestamp("time", 0, "").encode_value(value).hex(), "0382")
             )
-        elif setting_id == 'eco_mode_1':
-            await self._read_from_socket(Aa55ProtocolCommand("02390b070104" + value.hex(), "02b9"))
-        elif setting_id == 'eco_mode_2':
-            await self._read_from_socket(Aa55ProtocolCommand("02390b070504" + value.hex(), "02b9"))
-        elif setting_id == 'eco_mode_3':
-            await self._read_from_socket(Aa55ProtocolCommand("02390b070904" + value.hex(), "02b9"))
-        elif setting_id == 'eco_mode_4':
-            await self._read_from_socket(Aa55ProtocolCommand("02390b070d04" + value.hex(), "02b9"))
         else:
             raise InverterError("Operation not supported")
 
@@ -211,27 +200,43 @@ class ES(Inverter):
                 Aa55ProtocolCommand("033502" + "{:04x}".format(export_limit), "03b5")
             )
 
-    async def get_operation_mode(self) -> int:
-        return await self.read_setting('work_mode')
+    async def get_operation_modes(self, include_emulated: bool) -> Tuple[OperationMode, ...]:
+        result = [e for e in OperationMode]
+        result.remove(OperationMode.PEAK_SHAVING)
+        if not include_emulated:
+            result.remove(OperationMode.ECO_CHARGE)
+            result.remove(OperationMode.ECO_DISCHARGE)
+        return tuple(result)
 
-    async def set_operation_mode(self, operation_mode: int, eco_mode_power: int = 100) -> None:
-        if operation_mode == 0:
+    async def get_operation_mode(self) -> OperationMode:
+        mode = OperationMode(await self.read_setting('work_mode'))
+        if OperationMode.ECO != mode:
+            return mode
+        all_settings = await self.read_settings_data()
+        eco_charge = all_settings.get('eco_mode_charge')
+        eco_discharge = all_settings.get('eco_mode_discharge')
+        if eco_charge.is_eco_charge_mode():
+            return OperationMode.ECO_CHARGE
+        elif eco_discharge.is_eco_discharge_mode():
+            return OperationMode.ECO_DISCHARGE
+        else:
+            return OperationMode.ECO
+
+    async def set_operation_mode(self, operation_mode: OperationMode, eco_mode_power: int = 100) -> None:
+        if operation_mode == OperationMode.GENERAL:
             await self._set_general_mode()
-        elif operation_mode == 1:
+        elif operation_mode == OperationMode.OFF_GRID:
             await self._set_offgrid_mode()
-        elif operation_mode == 2:
+        elif operation_mode == OperationMode.BACKUP:
             await self._set_backup_mode()
-        elif operation_mode == 3:
-            await self._set_eco_mode(eco_mode_power)
-        elif operation_mode in (4, 5):
-            if operation_mode == 4:
-                await self.write_setting('eco_mode_1', EcoMode("1", 0, "").encode_charge(eco_mode_power))
-            else:
-                await self.write_setting('eco_mode_1', EcoMode("1", 0, "").encode_discharge(eco_mode_power))
-            await self.write_setting('eco_mode_2', EcoMode("2", 0, "").encode_off())
-            await self.write_setting('eco_mode_3', EcoMode("3", 0, "").encode_off())
-            await self.write_setting('eco_mode_4', EcoMode("4", 0, "").encode_off())
+        elif operation_mode == OperationMode.ECO:
             await self._set_eco_mode(eco_mode_power)
+        elif operation_mode == OperationMode.PEAK_SHAVING:
+            raise InverterError("Operation not supported.")
+        elif operation_mode == OperationMode.ECO_CHARGE:
+            await self._set_eco_mode(-1 * abs(eco_mode_power))
+        elif operation_mode == OperationMode.ECO_DISCHARGE:
+            await self._set_eco_mode(abs(eco_mode_power))
 
     async def get_ongrid_battery_dod(self) -> int:
         return await self.read_setting('dod')
@@ -252,7 +257,7 @@ class ES(Inverter):
         return self.__settings
 
     async def _set_general_mode(self) -> None:
-        if self.arm_sw_version >= 7:
+        if self.arm_version >= 7:
             if self._supports_new_eco_mode():
                 await self._clear_battery_mode_param()
             else:
@@ -266,7 +271,7 @@ class ES(Inverter):
         await self._set_work_mode(0)
 
     async def _set_offgrid_mode(self) -> None:
-        if self.arm_sw_version >= 7:
+        if self.arm_version >= 7:
             await self._clear_battery_mode_param()
         else:
             await self._set_limit_power_for_charge(0, 0, 23, 59, 0)
@@ -277,7 +282,7 @@ class ES(Inverter):
         await self._set_work_mode(1)
 
     async def _set_backup_mode(self) -> None:
-        if self.arm_sw_version >= 7:
+        if self.arm_version >= 7:
             if self._supports_new_eco_mode():
                 await self._clear_battery_mode_param()
             else:
@@ -290,8 +295,8 @@ class ES(Inverter):
         await self._set_work_mode(2)
 
     async def _set_eco_mode(self, eco_mode_power: int) -> None:
-        await self._set_limit_power_for_charge(0, 0, 23, 59, eco_mode_power)
-        await self._set_limit_power_for_discharge(0, 0, 23, 59, eco_mode_power)
+        await self._set_limit_power_for_charge(0, 0, 23, 59, abs(eco_mode_power) if eco_mode_power < 0 else 0)
+        await self._set_limit_power_for_discharge(0, 0, 23, 59, eco_mode_power if eco_mode_power > 0 else 0)
         await self._set_offgrid_work_mode(0)
         await self._set_work_mode(3)
 
@@ -319,11 +324,11 @@ class ES(Inverter):
         await self._read_from_socket(Aa55ProtocolCommand("033601" + "{:02x}".format(mode), "03B6"))
 
     def _supports_new_eco_mode(self) -> bool:
-        if self.arm_sw_version < 14:
+        if self.arm_version < 14:
             return False
-        if len(self.arm_version) < 2:
+        if len(self.arm_firmware) < 2:
             return False
-        fw_version = int(self.arm_version[0:2])
+        fw_version = int(self.arm_firmware[0:2])
         if "EMU" in self.serial_number:
             return fw_version >= 11
         if "ESU" in self.serial_number:
